@@ -1,28 +1,37 @@
-# -*- coding: utf-8 -*-
+from __future__ import annotations
+
 import ast
+import gzip
+import hashlib
 import shutil
 import tarfile
 
 from email.parser import Parser
+from pathlib import Path
+from typing import TYPE_CHECKING
+from typing import Any
+from typing import Iterator
 
 import pytest
 
+from packaging.utils import canonicalize_name
+
 from poetry.core.factory import Factory
 from poetry.core.masonry.builders.sdist import SdistBuilder
 from poetry.core.masonry.utils.package_include import PackageInclude
-from poetry.core.packages import Package
 from poetry.core.packages.dependency import Dependency
+from poetry.core.packages.project_package import ProjectPackage
 from poetry.core.packages.vcs_dependency import VCSDependency
-from poetry.core.utils._compat import Path
-from poetry.core.utils._compat import encode
-from poetry.core.utils._compat import to_str
 
 
+if TYPE_CHECKING:
+    from pytest_mock import MockerFixture
+
 fixtures_dir = Path(__file__).parent / "fixtures"
 
 
 @pytest.fixture(autouse=True)
-def setup():
+def setup() -> Iterator[None]:
     clear_samples_dist()
 
     yield
@@ -30,18 +39,18 @@ def setup():
     clear_samples_dist()
 
 
-def clear_samples_dist():
+def clear_samples_dist() -> None:
     for dist in fixtures_dir.glob("**/dist"):
         if dist.is_dir():
             shutil.rmtree(str(dist))
 
 
-def project(name):
+def project(name: str) -> Path:
     return Path(__file__).parent / "fixtures" / name
 
 
-def test_convert_dependencies():
-    package = Package("foo", "1.2.3")
+def test_convert_dependencies() -> None:
+    package = ProjectPackage("foo", "1.2.3")
     result = SdistBuilder.convert_dependencies(
         package,
         [
@@ -57,16 +66,16 @@ def test_convert_dependencies():
         "A>=1.0,<2.0",
         "B>=1.0,<1.1",
         "C==1.2.3",
-        "D @ git+https://github.com/sdispater/d.git@master",
+        "D @ git+https://github.com/sdispater/d.git",
         "E>=1.0,<2.0",
         "F>=1.0,<2.0,!=1.3",
     ]
-    extras = {}
+    extras: dict[str, Any] = {}
 
     assert result == (main, extras)
 
-    package = Package("foo", "1.2.3")
-    package.extras = {"bar": [Dependency("A", "*")]}
+    package = ProjectPackage("foo", "1.2.3")
+    package.extras = {canonicalize_name("bar"): [Dependency("A", "*")]}
 
     result = SdistBuilder.convert_dependencies(
         package,
@@ -86,7 +95,7 @@ def test_convert_dependencies():
     d = Dependency("D", "3.4.5", optional=True)
     d.python_versions = "~2.7 || ^3.4"
 
-    package.extras = {"baz": [Dependency("D", "*")]}
+    package.extras = {canonicalize_name("baz"): [Dependency("D", "*")]}
 
     result = SdistBuilder.convert_dependencies(
         package,
@@ -107,7 +116,7 @@ def test_convert_dependencies():
     assert result == (main, extras)
 
 
-def test_make_setup():
+def test_make_setup() -> None:
     poetry = Factory().create_poetry(project("complete"))
 
     builder = SdistBuilder(poetry)
@@ -115,7 +124,7 @@ def test_make_setup():
     setup_ast = ast.parse(setup)
 
     setup_ast.body = [n for n in setup_ast.body if isinstance(n, ast.Assign)]
-    ns = {}
+    ns: dict[str, Any] = {}
     exec(compile(setup_ast, filename="setup.py", mode="exec"), ns)
     assert ns["packages"] == [
         "my_package",
@@ -131,6 +140,7 @@ def test_make_setup():
             "my-script = my_package:main",
         ]
     }
+    assert ns["scripts"] == [str(Path("bin") / "script.sh")]
     assert ns["extras_require"] == {
         'time:python_version ~= "2.7" and sys_platform == "win32" or python_version in "3.4 3.5"': [
             "pendulum>=1.4,<2.0"
@@ -138,7 +148,7 @@ def test_make_setup():
     }
 
 
-def test_make_pkg_info(mocker):
+def test_make_pkg_info(mocker: MockerFixture) -> None:
     get_metadata_content = mocker.patch(
         "poetry.core.masonry.builders.builder.Builder.get_metadata_content"
     )
@@ -150,18 +160,18 @@ def test_make_pkg_info(mocker):
     assert get_metadata_content.called
 
 
-def test_make_pkg_info_any_python():
+def test_make_pkg_info_any_python() -> None:
     poetry = Factory().create_poetry(project("module1"))
 
     builder = SdistBuilder(poetry)
     pkg_info = builder.build_pkg_info()
     p = Parser()
-    parsed = p.parsestr(to_str(pkg_info))
+    parsed = p.parsestr(pkg_info.decode())
 
     assert "Requires-Python" not in parsed
 
 
-def test_find_files_to_add():
+def test_find_files_to_add() -> None:
     poetry = Factory().create_poetry(project("complete"))
 
     builder = SdistBuilder(poetry)
@@ -171,6 +181,7 @@ def test_find_files_to_add():
         [
             Path("LICENSE"),
             Path("README.rst"),
+            Path("bin/script.sh"),
             Path("my_package/__init__.py"),
             Path("my_package/data1/test.json"),
             Path("my_package/sub_pkg1/__init__.py"),
@@ -182,7 +193,7 @@ def test_find_files_to_add():
     )
 
 
-def test_make_pkg_info_multi_constraints_dependency():
+def test_make_pkg_info_multi_constraints_dependency() -> None:
     poetry = Factory().create_poetry(
         Path(__file__).parent.parent.parent
         / "fixtures"
@@ -192,7 +203,7 @@ def test_make_pkg_info_multi_constraints_dependency():
     builder = SdistBuilder(poetry)
     pkg_info = builder.build_pkg_info()
     p = Parser()
-    parsed = p.parsestr(to_str(pkg_info))
+    parsed = p.parsestr(pkg_info.decode())
 
     requires = parsed.get_all("Requires-Dist")
     assert requires == [
@@ -201,7 +212,7 @@ def test_make_pkg_info_multi_constraints_dependency():
     ]
 
 
-def test_find_packages():
+def test_find_packages() -> None:
     poetry = Factory().create_poetry(project("complete"))
 
     builder = SdistBuilder(poetry)
@@ -238,21 +249,39 @@ def test_find_packages():
     assert pkg_data == {"": ["*"]}
 
 
-def test_package():
+def test_package() -> None:
     poetry = Factory().create_poetry(project("complete"))
 
     builder = SdistBuilder(poetry)
     builder.build()
 
-    sdist = fixtures_dir / "complete" / "dist" / "my-package-1.2.3.tar.gz"
+    sdist = fixtures_dir / "complete" / "dist" / "my_package-1.2.3.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
-        assert "my-package-1.2.3/LICENSE" in tar.getnames()
+        assert "my_package-1.2.3/LICENSE" in tar.getnames()
 
 
-def test_setup_py_context():
+def test_sdist_reproducibility() -> None:
+    poetry = Factory().create_poetry(project("complete"))
+
+    hashes = set()
+
+    for _ in range(2):
+        builder = SdistBuilder(poetry)
+        builder.build()
+
+        sdist = fixtures_dir / "complete" / "dist" / "my_package-1.2.3.tar.gz"
+
+        assert sdist.exists()
+
+        hashes.add(hashlib.sha256(sdist.read_bytes()).hexdigest())
+
+    assert len(hashes) == 1
+
+
+def test_setup_py_context() -> None:
     poetry = Factory().create_poetry(project("complete"))
 
     builder = SdistBuilder(poetry)
@@ -268,8 +297,8 @@ def test_setup_py_context():
 
             with open(str(setup), "rb") as f:
                 # we convert to string  and replace line endings here for compatibility
-                data = to_str(encode(f.read())).replace("\r\n", "\n")
-                assert data == to_str(builder.build_setup())
+                data = f.read().decode().replace("\r\n", "\n")
+                assert data == builder.build_setup().decode()
 
         assert not project_setup_py.exists()
     finally:
@@ -277,7 +306,7 @@ def test_setup_py_context():
             project_setup_py.unlink()
 
 
-def test_module():
+def test_module() -> None:
     poetry = Factory().create_poetry(project("module1"))
 
     builder = SdistBuilder(poetry)
@@ -291,7 +320,7 @@ def test_module():
         assert "module1-0.1/module1.py" in tar.getnames()
 
 
-def test_prelease():
+def test_prelease() -> None:
     poetry = Factory().create_poetry(project("prerelease"))
 
     builder = SdistBuilder(poetry)
@@ -302,8 +331,8 @@ def test_prelease():
     assert sdist.exists()
 
 
-@pytest.mark.parametrize("directory", [("extended"), ("extended_legacy_config")])
-def test_with_c_extensions(directory):
+@pytest.mark.parametrize("directory", ["extended", "extended_legacy_config"])
+def test_with_c_extensions(directory: str) -> None:
     poetry = Factory().create_poetry(project("extended"))
 
     builder = SdistBuilder(poetry)
@@ -318,7 +347,7 @@ def test_with_c_extensions(directory):
         assert "extended-0.1/extended/extended.c" in tar.getnames()
 
 
-def test_with_c_extensions_src_layout():
+def test_with_c_extensions_src_layout() -> None:
     poetry = Factory().create_poetry(project("src_extended"))
 
     builder = SdistBuilder(poetry)
@@ -333,7 +362,16 @@ def test_with_c_extensions_src_layout():
         assert "extended-0.1/src/extended/extended.c" in tar.getnames()
 
 
-def test_with_src_module_file():
+def test_with_build_script_in_subdir() -> None:
+    poetry = Factory().create_poetry(project("build_script_in_subdir"))
+
+    builder = SdistBuilder(poetry)
+    setup = builder.build_setup()
+    # should not error
+    ast.parse(setup)
+
+
+def test_with_src_module_file() -> None:
     poetry = Factory().create_poetry(project("source_file"))
 
     builder = SdistBuilder(poetry)
@@ -343,22 +381,22 @@ def test_with_src_module_file():
     setup_ast = ast.parse(setup)
 
     setup_ast.body = [n for n in setup_ast.body if isinstance(n, ast.Assign)]
-    ns = {}
+    ns: dict[str, Any] = {}
     exec(compile(setup_ast, filename="setup.py", mode="exec"), ns)
     assert ns["package_dir"] == {"": "src"}
     assert ns["modules"] == ["module_src"]
 
     builder.build()
 
-    sdist = fixtures_dir / "source_file" / "dist" / "module-src-0.1.tar.gz"
+    sdist = fixtures_dir / "source_file" / "dist" / "module_src-0.1.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
-        assert "module-src-0.1/src/module_src.py" in tar.getnames()
+        assert "module_src-0.1/src/module_src.py" in tar.getnames()
 
 
-def test_with_src_module_dir():
+def test_with_src_module_dir() -> None:
     poetry = Factory().create_poetry(project("source_package"))
 
     builder = SdistBuilder(poetry)
@@ -368,40 +406,44 @@ def test_with_src_module_dir():
     setup_ast = ast.parse(setup)
 
     setup_ast.body = [n for n in setup_ast.body if isinstance(n, ast.Assign)]
-    ns = {}
+    ns: dict[str, Any] = {}
     exec(compile(setup_ast, filename="setup.py", mode="exec"), ns)
     assert ns["package_dir"] == {"": "src"}
     assert ns["packages"] == ["package_src"]
 
     builder.build()
 
-    sdist = fixtures_dir / "source_package" / "dist" / "package-src-0.1.tar.gz"
+    sdist = fixtures_dir / "source_package" / "dist" / "package_src-0.1.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
-        assert "package-src-0.1/src/package_src/__init__.py" in tar.getnames()
-        assert "package-src-0.1/src/package_src/module.py" in tar.getnames()
+        assert "package_src-0.1/src/package_src/__init__.py" in tar.getnames()
+        assert "package_src-0.1/src/package_src/module.py" in tar.getnames()
 
 
-def test_default_with_excluded_data(mocker):
-    # Patch git module to return specific excluded files
-    p = mocker.patch("poetry.core.vcs.git.Git.get_ignored_files")
-    p.return_value = [
-        (
-            (
-                Path(__file__).parent
-                / "fixtures"
-                / "default_with_excluded_data"
-                / "my_package"
-                / "data"
-                / "sub_data"
-                / "data2.txt"
-            )
-            .relative_to(project("default_with_excluded_data"))
-            .as_posix()
-        )
-    ]
+def test_default_with_excluded_data(mocker: MockerFixture) -> None:
+    class MockGit:
+        def get_ignored_files(self, folder: Path | None = None) -> list[str]:
+            # Patch git module to return specific excluded files
+            return [
+                (
+                    (
+                        Path(__file__).parent
+                        / "fixtures"
+                        / "default_with_excluded_data"
+                        / "my_package"
+                        / "data"
+                        / "sub_data"
+                        / "data2.txt"
+                    )
+                    .relative_to(project("default_with_excluded_data"))
+                    .as_posix()
+                )
+            ]
+
+    p = mocker.patch("poetry.core.vcs.get_vcs")
+    p.return_value = MockGit()
     poetry = Factory().create_poetry(project("default_with_excluded_data"))
 
     builder = SdistBuilder(poetry)
@@ -411,7 +453,7 @@ def test_default_with_excluded_data(mocker):
     setup_ast = ast.parse(setup)
 
     setup_ast.body = [n for n in setup_ast.body if isinstance(n, ast.Assign)]
-    ns = {}
+    ns: dict[str, Any] = {}
     exec(compile(setup_ast, filename="setup.py", mode="exec"), ns)
     assert "package_dir" not in ns
     assert ns["packages"] == ["my_package"]
@@ -423,7 +465,7 @@ def test_default_with_excluded_data(mocker):
     builder.build()
 
     sdist = (
-        fixtures_dir / "default_with_excluded_data" / "dist" / "my-package-1.2.3.tar.gz"
+        fixtures_dir / "default_with_excluded_data" / "dist" / "my_package-1.2.3.tar.gz"
     )
 
     assert sdist.exists()
@@ -431,90 +473,97 @@ def test_default_with_excluded_data(mocker):
     with tarfile.open(str(sdist), "r") as tar:
         names = tar.getnames()
         assert len(names) == len(set(names))
-        assert "my-package-1.2.3/LICENSE" in names
-        assert "my-package-1.2.3/README.rst" in names
-        assert "my-package-1.2.3/my_package/__init__.py" in names
-        assert "my-package-1.2.3/my_package/data/data1.txt" in names
-        assert "my-package-1.2.3/pyproject.toml" in names
-        assert "my-package-1.2.3/setup.py" in names
-        assert "my-package-1.2.3/PKG-INFO" in names
+        assert "my_package-1.2.3/LICENSE" in names
+        assert "my_package-1.2.3/README.rst" in names
+        assert "my_package-1.2.3/my_package/__init__.py" in names
+        assert "my_package-1.2.3/my_package/data/data1.txt" in names
+        assert "my_package-1.2.3/pyproject.toml" in names
+        assert "my_package-1.2.3/setup.py" in names
+        assert "my_package-1.2.3/PKG-INFO" in names
         # all last modified times should be set to a valid timestamp
         for tarinfo in tar.getmembers():
-            assert 0 < tarinfo.mtime
+            if tarinfo.name in [
+                "my_package-1.2.3/setup.py",
+                "my_package-1.2.3/PKG-INFO",
+            ]:
+                # generated files have timestamp set to 0
+                assert tarinfo.mtime == 0
+                continue
+            assert tarinfo.mtime > 0
 
 
-def test_src_excluded_nested_data():
+def test_src_excluded_nested_data() -> None:
     module_path = fixtures_dir / "exclude_nested_data_toml"
     poetry = Factory().create_poetry(module_path)
 
     builder = SdistBuilder(poetry)
     builder.build()
 
-    sdist = module_path / "dist" / "my-package-1.2.3.tar.gz"
+    sdist = module_path / "dist" / "my_package-1.2.3.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
         names = tar.getnames()
         assert len(names) == len(set(names))
-        assert "my-package-1.2.3/LICENSE" in names
-        assert "my-package-1.2.3/README.rst" in names
-        assert "my-package-1.2.3/pyproject.toml" in names
-        assert "my-package-1.2.3/setup.py" in names
-        assert "my-package-1.2.3/PKG-INFO" in names
-        assert "my-package-1.2.3/my_package/__init__.py" in names
-        assert "my-package-1.2.3/my_package/data/sub_data/data2.txt" not in names
-        assert "my-package-1.2.3/my_package/data/sub_data/data3.txt" not in names
-        assert "my-package-1.2.3/my_package/data/data1.txt" not in names
-        assert "my-package-1.2.3/my_package/data/data2.txt" in names
-        assert "my-package-1.2.3/my_package/puplic/publicdata.txt" in names
-        assert "my-package-1.2.3/my_package/public/item1/itemdata1.txt" not in names
+        assert "my_package-1.2.3/LICENSE" in names
+        assert "my_package-1.2.3/README.rst" in names
+        assert "my_package-1.2.3/pyproject.toml" in names
+        assert "my_package-1.2.3/setup.py" in names
+        assert "my_package-1.2.3/PKG-INFO" in names
+        assert "my_package-1.2.3/my_package/__init__.py" in names
+        assert "my_package-1.2.3/my_package/data/sub_data/data2.txt" not in names
+        assert "my_package-1.2.3/my_package/data/sub_data/data3.txt" not in names
+        assert "my_package-1.2.3/my_package/data/data1.txt" not in names
+        assert "my_package-1.2.3/my_package/data/data2.txt" in names
+        assert "my_package-1.2.3/my_package/puplic/publicdata.txt" in names
+        assert "my_package-1.2.3/my_package/public/item1/itemdata1.txt" not in names
         assert (
-            "my-package-1.2.3/my_package/public/item1/subitem/subitemdata.txt"
+            "my_package-1.2.3/my_package/public/item1/subitem/subitemdata.txt"
             not in names
         )
-        assert "my-package-1.2.3/my_package/public/item2/itemdata2.txt" not in names
+        assert "my_package-1.2.3/my_package/public/item2/itemdata2.txt" not in names
 
 
-def test_proper_python_requires_if_two_digits_precision_version_specified():
+def test_proper_python_requires_if_two_digits_precision_version_specified() -> None:
     poetry = Factory().create_poetry(project("simple_version"))
 
     builder = SdistBuilder(poetry)
     pkg_info = builder.build_pkg_info()
     p = Parser()
-    parsed = p.parsestr(to_str(pkg_info))
+    parsed = p.parsestr(pkg_info.decode())
 
     assert parsed["Requires-Python"] == ">=3.6,<3.7"
 
 
-def test_proper_python_requires_if_three_digits_precision_version_specified():
+def test_proper_python_requires_if_three_digits_precision_version_specified() -> None:
     poetry = Factory().create_poetry(project("single_python"))
 
     builder = SdistBuilder(poetry)
     pkg_info = builder.build_pkg_info()
     p = Parser()
-    parsed = p.parsestr(to_str(pkg_info))
+    parsed = p.parsestr(pkg_info.decode())
 
     assert parsed["Requires-Python"] == "==2.7.15"
 
 
-def test_includes():
+def test_includes() -> None:
     poetry = Factory().create_poetry(project("with-include"))
 
     builder = SdistBuilder(poetry)
 
     builder.build()
 
-    sdist = fixtures_dir / "with-include" / "dist" / "with-include-1.2.3.tar.gz"
+    sdist = fixtures_dir / "with-include" / "dist" / "with_include-1.2.3.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
-        assert "with-include-1.2.3/extra_dir/vcs_excluded.txt" in tar.getnames()
-        assert "with-include-1.2.3/notes.txt" in tar.getnames()
+        assert "with_include-1.2.3/extra_dir/vcs_excluded.txt" in tar.getnames()
+        assert "with_include-1.2.3/notes.txt" in tar.getnames()
 
 
-def test_includes_with_inline_table():
+def test_includes_with_inline_table() -> None:
     poetry = Factory().create_poetry(project("with_include_inline_table"))
 
     builder = SdistBuilder(poetry)
@@ -525,18 +574,19 @@ def test_includes_with_inline_table():
         fixtures_dir
         / "with_include_inline_table"
         / "dist"
-        / "with-include-1.2.3.tar.gz"
+        / "with_include-1.2.3.tar.gz"
     )
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
-        assert "with-include-1.2.3/wheel_only.txt" not in tar.getnames()
-        assert "with-include-1.2.3/tests/__init__.py" in tar.getnames()
-        assert "with-include-1.2.3/tests/test_foo/test.py" in tar.getnames()
+        assert "with_include-1.2.3/both.txt" in tar.getnames()
+        assert "with_include-1.2.3/wheel_only.txt" not in tar.getnames()
+        assert "with_include-1.2.3/tests/__init__.py" in tar.getnames()
+        assert "with_include-1.2.3/tests/test_foo/test.py" in tar.getnames()
 
 
-def test_excluded_subpackage():
+def test_excluded_subpackage() -> None:
     poetry = Factory().create_poetry(project("excluded_subpackage"))
 
     builder = SdistBuilder(poetry)
@@ -545,45 +595,76 @@ def test_excluded_subpackage():
     setup_ast = ast.parse(setup)
 
     setup_ast.body = [n for n in setup_ast.body if isinstance(n, ast.Assign)]
-    ns = {}
+    ns: dict[str, Any] = {}
     exec(compile(setup_ast, filename="setup.py", mode="exec"), ns)
 
     assert ns["packages"] == ["example"]
 
 
-def test_sdist_package_pep_561_stub_only():
+def test_sdist_package_pep_561_stub_only() -> None:
     root = fixtures_dir / "pep_561_stub_only"
     poetry = Factory().create_poetry(root)
 
     builder = SdistBuilder(poetry)
     builder.build()
 
-    sdist = root / "dist" / "pep-561-stubs-0.1.tar.gz"
+    sdist = root / "dist" / "pep_561_stubs-0.1.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
         names = tar.getnames()
-        assert "pep-561-stubs-0.1/pkg-stubs/__init__.pyi" in names
-        assert "pep-561-stubs-0.1/pkg-stubs/module.pyi" in names
-        assert "pep-561-stubs-0.1/pkg-stubs/subpkg/__init__.pyi" in names
+        assert "pep_561_stubs-0.1/pkg-stubs/__init__.pyi" in names
+        assert "pep_561_stubs-0.1/pkg-stubs/module.pyi" in names
+        assert "pep_561_stubs-0.1/pkg-stubs/subpkg/__init__.pyi" in names
 
 
-def test_sdist_disable_setup_py():
+def test_sdist_disable_setup_py() -> None:
     module_path = fixtures_dir / "disable_setup_py"
     poetry = Factory().create_poetry(module_path)
 
     builder = SdistBuilder(poetry)
     builder.build()
 
-    sdist = module_path / "dist" / "my-package-1.2.3.tar.gz"
+    sdist = module_path / "dist" / "my_package-1.2.3.tar.gz"
 
     assert sdist.exists()
 
     with tarfile.open(str(sdist), "r") as tar:
         assert set(tar.getnames()) == {
-            "my-package-1.2.3/README.rst",
-            "my-package-1.2.3/pyproject.toml",
-            "my-package-1.2.3/PKG-INFO",
-            "my-package-1.2.3/my_package/__init__.py",
+            "my_package-1.2.3/README.rst",
+            "my_package-1.2.3/pyproject.toml",
+            "my_package-1.2.3/PKG-INFO",
+            "my_package-1.2.3/my_package/__init__.py",
         }
+
+
+def test_sdist_mtime_zero() -> None:
+    poetry = Factory().create_poetry(project("module1"))
+
+    builder = SdistBuilder(poetry)
+    builder.build()
+
+    sdist = fixtures_dir / "module1" / "dist" / "module1-0.1.tar.gz"
+
+    assert sdist.exists()
+
+    with gzip.open(str(sdist), "rb") as gz:
+        gz.read(100)
+        assert gz.mtime == 0
+
+
+def test_split_source() -> None:
+    root = fixtures_dir / "split_source"
+    poetry = Factory().create_poetry(root)
+
+    builder = SdistBuilder(poetry)
+
+    # Check setup.py
+    setup = builder.build_setup()
+    setup_ast = ast.parse(setup)
+
+    setup_ast.body = [n for n in setup_ast.body if isinstance(n, ast.Assign)]
+    ns: dict[str, Any] = {}
+    exec(compile(setup_ast, filename="setup.py", mode="exec"), ns)
+    assert "" in ns["package_dir"] and "module_b" in ns["package_dir"]
