@@ -7,10 +7,12 @@ from typing import cast
 
 import pytest
 
+from packaging.utils import canonicalize_name
+
+from poetry.core.constraints.version import parse_constraint
 from poetry.core.factory import Factory
 from poetry.core.packages.url_dependency import URLDependency
 from poetry.core.packages.vcs_dependency import VCSDependency
-from poetry.core.semver.helpers import parse_constraint
 from poetry.core.toml import TOMLFile
 from poetry.core.version.markers import SingleMarker
 
@@ -141,6 +143,7 @@ def test_create_poetry() -> None:
         "Programming Language :: Python :: 3.8",
         "Programming Language :: Python :: 3.9",
         "Programming Language :: Python :: 3.10",
+        "Programming Language :: Python :: 3.11",
         "Topic :: Software Development :: Build Tools",
         "Topic :: Software Development :: Libraries :: Python Modules",
     ]
@@ -342,7 +345,7 @@ def test_create_poetry_with_markers_and_extras() -> None:
 
     assert len(dependencies) == 2
     assert {dependency.name for dependency in dependencies} == {"orjson"}
-    assert set(extras["all"]) == set(dependencies)
+    assert set(extras[canonicalize_name("all")]) == set(dependencies)
     for dependency in dependencies:
         assert dependency.in_extras == ["all"]
         assert isinstance(dependency, URLDependency)
